/*
  fan.cpp
*/

#include <iostream>
#include <sdpsl.h>

using namespace std;
using namespace sdp;

//
// Prototypes
//
static void output_fan_sdp(const Matrix<double>& A, int k);

/*
  main
  
  We apply function output_fan_sdp to a sample matrix.
*/
int main()
{
  Matrix<double> A({ { -2,  4,  3, -5, -1, -6 }, 
      	             {  4,  8,  0,  0,  6,  3 }, 
		     {  3,  0,  0,  0, -5,  3 }, 
		     { -5,  0,  0, -2, -2, -4 }, 
		     { -1,  6, -5, -2, -2,  1 }, 
		     { -6,  3,  3, -4,  1, -2 } });
  
  output_fan_sdp(A, 3);  // sum of 3 largest eigenvalues

  return 0;
}

/*
  output_fan_sdp

  Outputs an SDP whose optimal value is the sum of the k largest
  eigenvalues of symmetric matrix A.
*/
void output_fan_sdp(const Matrix<double>& A, int k)
{
  int n = A.num_rows();
  SDPVariable X(n), Y(n);
  SDPConstraintContainer cc;

  /* First the trace constraint for X */
  cc.add_constraint(Matrix<double>::identity(n) * X == k);

  /* Then the correspondence between X and Y */
  Matrix<double> E(n, n, 1);
  MatrixMask E_mask(n);

  for (int i = 0; i < n; i++)
    for (int j = i; j < n; j++) {
      double rhs = i == j ? 1 : 0;

      E_mask.turn_on(i, j);
      cc.add_constraint(SDPMaskedMatrix(E, E_mask) * X
			+ SDPMaskedMatrix(E, E_mask) * Y == rhs);
      E_mask.turn_off(i, j);      
    }

  /* Then we create the problem and output it */
  SDPProblem problem;

  problem.set_objective(A * X);
  problem << cc;

  cout << SDPAWriter(problem);
}
