/*
  theta.cpp

  Outputs, in sparse SDPA format, the semidefinite programming problem
  corresponding to the Lovász theta number of a pentagon.

  Copyright (c) 2014 Fernando M. de Oliveira Filho
  <http://www.ime.usp.br/~fmario>
  
  This file is part of SDPSL.
  
  SDPSL is free software: you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your
  option) any later version.
  
  SDPSL is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  for more details.
  
  You should have received a copy of the GNU General Public License
  along with SDPSL.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <iostream>
#include <sdpsl.h>

using namespace std;
using namespace sdp;

int main()
{
  SDPVariable X(5);
  SDPConstraintContainer cc;

  /* We first generate one constraint for each edge of the 5-cycle,
     using masks for efficiency. 
  */
  MatrixMask mask(5);
  Matrix<double> J(5, 5, 1);  // A 5 x 5 all-ones matrix
  SDPLinearConstraint edge_constraint = SDPMaskedMatrix(J, mask) * X == 0;

  for (int i = 0; i < 5; i++) {
    int j = (i + 1) % 5;

    mask.turn_on(i, j);
    cc.add_constraint(edge_constraint);
    mask.turn_off(i, j);
  }

  /* The trace == 1 constraint */
  cc.add_constraint(Matrix<double>::identity(5) * X == 1);

  /* We add the constraints to the problem */
  SDPProblem problem;

  problem << cc;

  /* We set the objective function */
  problem.set_objective(J * X);

  /* And we output the SDPA file */
  cout << SDPAWriter(problem);

  return 0;
}
