/*
  tensor.h

  TODO: check because of changes to Matrix

  Copyright (c) 2014 Fernando M. de Oliveira Filho
  <http://www.ime.usp.br/~fmario>
  
  This file is part of SDPSL.
  
  SDPSL is free software: you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your
  option) any later version.
  
  SDPSL is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  for more details.
  
  You should have received a copy of the GNU General Public License
  along with SDPSL.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __TENSOR_H__
#define __TENSOR_H__

#include <sdpsl/matrix.h>
#include <sdpsl/tensortemp.h>

#ifdef __MPF_BUILD__
namespace mpf_sdp {
#else
namespace sdp {
#endif

//
// Class declaration
//
template <class T>
class Tensor : public Matrix<T> {
  TensorTemplate temp;

public:
  explicit Tensor(const TensorTemplate&, const T& = T());
  explicit Tensor(const std::initializer_list<DimBounds>&, const T& = T());

  /* Get and set */
  const TensorTemplate& tensor_template() const { return temp; }
  int size() const { return temp.matrix_size(); }
  int idx(const int *p) const { return temp.idx(p); }
  
  /* Iterator */
  typedef TensorTemplate::iterator iterator;

  iterator begin() const { return temp.begin(); }
  iterator end() const { return temp.end(); }

  /* Indexing operator - nonconstant */
  class TensorRow {
    friend class Tensor<T>;

    int row;
    Matrix<T>& mat;
    const TensorTemplate& temp;

    TensorRow(int i, Matrix<T>& m, const TensorTemplate& t) : row(i), mat(m), temp(t) { }

  public:
    T& operator[](int j) { return mat[row][j]; }
    T& operator[](const int *l) { return mat[row][temp.idx(l)]; }
    T& operator[](const iterator& it) { return mat[row][it.idx()]; }
  };

  TensorRow operator[](int i) { return TensorRow(i, *this, temp); }
  TensorRow operator[](const int *l) { return TensorRow(temp.idx(l), *this, temp); }
  TensorRow operator[](const iterator& it) { return TensorRow(it.idx(), *this, temp); }

  /* Indexing operator - constant */
  class ConstTensorRow {
    friend class Tensor<T>;

    int row;
    const Matrix<T>& mat;
    const TensorTemplate& temp;

    ConstTensorRow(int i, const Matrix<T>& m, const TensorTemplate& t) : row(i), mat(m), temp(t) { }

  public:
    const T& operator[](int j) const { return mat[row][j]; }
    const T& operator[](const int *l) const { return mat[row][temp.idx(l)]; }
    const T& operator[](const TensorTemplate::iterator& it) const { return mat[row][it.idx()]; }
  };

  ConstTensorRow operator[](int i) const { return ConstTensorRow(i, *this, temp); }
  ConstTensorRow operator[](const int *l) const { return ConstTensorRow(temp.idx(l), *this, temp); }
  ConstTensorRow operator[](const iterator& it) const { return ConstTensorRow(it.idx(), *this, temp); }
};

//
// Method definition
//
template <class T>
Tensor<T>::Tensor(const TensorTemplate& temp_, const T& val)
  : Matrix<T>(temp_.matrix_size(), temp_.matrix_size(), val), 
    temp(temp_)
{

}

template <class T>
Tensor<T>::Tensor(const std::initializer_list<DimBounds>& bounds, const T& val)
  : Matrix<T>(TensorTemplate(bounds).matrix_size(), TensorTemplate(bounds).matrix_size(), val), 
    temp(TensorTemplate(bounds))
{

}

} // namespace

#endif

// Local variables:
// mode: c++
// End:
