cdef extern from "sagesdpsl.h":
    cdef int FILE_ERROR 
    cdef int VALUE_ERROR
    cdef int INCOMPATIBLE_BASES_ERROR
    cdef int SINGULAR_MATRIX_ERROR
    cdef int INFEASIBLE_CONSTRAINT_ERROR
    cdef int REPRESENTATION_ERROR
    cdef int READ_ERROR
    cdef int INFEASIBLE_SYSTEM

    #
    # Environment variables
    #
    void set_gmp_prec(int prec)
    void set_output_eps(double eps)
    void set_tolerance_eps(double eps)

    #
    # Numbers
    #
    ctypedef void * MPFRNumber

    #
    # Polynomials
    #
    ctypedef void * SDPSLPoly

    int get_num_variables()
    int get_poly_size(SDPSLPoly p)
    void add_poly_variable(const char *name)
    SDPSLPoly create_polynomial()
    void destroy_polynomial(SDPSLPoly p)
    void add_monomial(SDPSLPoly p, int n, int *exponents, MPFRNumber coeff, int prec)
    void polynomial_to_array(SDPSLPoly p, int *n, int *ncoeffs, int **exponents, 
                             MPFRNumber **coeffs, int prec)

    #
    # Numerical matrices
    #
    ctypedef void * NumberMatrix

    NumberMatrix create_number_matrix(int m, int n, int prec, MPFRNumber init)
    void destroy_number_matrix(NumberMatrix A)
    void get_number_entry(NumberMatrix A, int i, int j, MPFRNumber num)
    void set_number_entry(NumberMatrix A, int i, int j, MPFRNumber num)
    void set_number_entries(NumberMatrix A, MPFRNumber *entries)

    #
    # Matrices of polynomials
    #
    ctypedef void * PolyMatrix

    PolyMatrix create_poly_matrix(int m, int n)
    void destroy_poly_matrix(PolyMatrix A)
    void get_poly_entry(PolyMatrix A, int i, int j, SDPSLPoly p)
    void set_poly_entry(PolyMatrix A, int i, int j, SDPSLPoly p)

    #
    # MatrixMask
    #
    ctypedef void * SDPSLMatrixMask

    SDPSLMatrixMask create_matrix_mask(int m, int n)
    void destroy_matrix_mask(SDPSLMatrixMask mask)
    int matrix_mask_is_on(SDPSLMatrixMask mask, int i, int j)
    void matrix_mask_turn_on(SDPSLMatrixMask mask, int i, int j)
    void matrix_mask_turn_off(SDPSLMatrixMask mask, int i, int j)
    void clear_matrix_mask(SDPSLMatrixMask mask)

    #
    # SDPVariable
    #
    ctypedef void * SDPSLVariable

    SDPSLVariable create_variable(int size)
    void destroy_variable(SDPSLVariable var)
    int get_variable_idx(SDPSLVariable var)

    #
    # SDPLinearCombination
    #
    ctypedef void * SDPSLLinearCombination

    SDPSLLinearCombination create_linear_combination()
    void destroy_linear_combination(SDPSLLinearCombination C)
    void linear_combination_add_matrix(SDPSLLinearCombination C, SDPSLVariable X,
                                       NumberMatrix A)
    void linear_combination_add_matrix_mask(SDPSLLinearCombination C, SDPSLVariable X,
                                            NumberMatrix A, SDPSLMatrixMask mask)

    #
    # SDPLinearConstraint
    #
    ctypedef void * SDPSLLinearConstraint

    SDPSLLinearConstraint create_linear_constraint(SDPSLLinearCombination lhs, char rel,
                                                   MPFRNumber rhs)
    void destroy_linear_constraint(SDPSLLinearConstraint C)
    void linear_constraint_set_rhs(SDPSLLinearConstraint C, MPFRNumber rhs)

    #
    # SDPPolynomialCombination
    #
    ctypedef void * SDPSLPolynomialCombination

    SDPSLPolynomialCombination create_poly_combination()
    void destroy_poly_combination(SDPSLPolynomialCombination C)
    void poly_combination_add_matrix(SDPSLPolynomialCombination C, SDPSLVariable X,
                                     PolyMatrix A)
    void poly_combination_add_matrix_mask(SDPSLPolynomialCombination C, SDPSLVariable X,
                                          PolyMatrix A, SDPSLMatrixMask mask)

    #
    # SDPPolynomialConstraint
    #
    ctypedef void * SDPSLPolynomialConstraint

    SDPSLPolynomialConstraint create_poly_constraint(SDPSLPolynomialCombination lhs, 
                                                     SDPSLPoly p)
    void destroy_poly_constraint(SDPSLPolynomialConstraint C)

    #
    # SparseMonomialBasis
    #
    ctypedef void * SDPSLSparseMonomialBasis

    SDPSLSparseMonomialBasis create_s_monomial_basis()
    SDPSLSparseMonomialBasis create_s_monomial_basis_cons(SDPSLPolynomialConstraint C)
    void destroy_s_monomial_basis(SDPSLSparseMonomialBasis basis)
    void s_monomial_basis_add(SDPSLSparseMonomialBasis basis, SDPSLPoly p)

    #
    # SparseBasisChanger
    #
    ctypedef void * SDPSLSparseBasisChanger

    SDPSLSparseBasisChanger create_s_basis_changer(SDPSLSparseMonomialBasis mon_basis,
                                                   int n, SDPSLPoly *poly_basis, int *error)
    void destroy_s_basis_changer(SDPSLSparseBasisChanger changer)

    #
    # SDPConstraintContainer
    #
    ctypedef void * SDPSLConstraintContainer

    SDPSLConstraintContainer create_constraint_container()
    void destroy_constraint_container(SDPSLConstraintContainer cc)
    int cc_add_linear(SDPSLConstraintContainer cc, SDPSLLinearConstraint constraint)
    int cc_add_poly(SDPSLConstraintContainer cc, SDPSLPolynomialConstraint constraint,
                    int progress, const char *msg)
    int cc_add_poly_with_basis(SDPSLConstraintContainer cc,
                               SDPSLPolynomialConstraint constraint,
                               SDPSLSparseBasisChanger changer,
                               int progress, const char *msg)
    int cc_get_num_constraints(SDPSLConstraintContainer cc)
    void cc_clear(SDPSLConstraintContainer cc)

    #
    # SDPProblem
    #
    ctypedef void * SDPSLProblem

    SDPSLProblem create_problem()
    void destroy_problem(SDPSLProblem problem)
    void problem_set_objective(SDPSLProblem problem, SDPSLLinearCombination C)
    void problem_add_container(SDPSLProblem problem, SDPSLConstraintContainer cc)

    #
    # SDPAWriter
    #
    int write_sdpa(SDPSLProblem problem, const char *filename, int prec)

    #
    # SDPSolution
    #
    ctypedef void * SDPSLSolution

    SDPSLSolution create_solution()
    void destroy_solution(SDPSLSolution sol)
    void solution_add_matrix(SDPSLSolution sol, SDPSLVariable X, NumberMatrix A)

    #
    # CSDPReader
    #
    int csdp_problem_read_solution(SDPSLProblem problem, const char *filename,
                                   SDPSLSolution sol, int *error_line)
    int csdp_list_read_solution(SDPSLVariable *vars, const char *filename,
                                SDPSLSolution sol, int *error_line)

    #
    # SDPSolutionAnalyzer
    #
    void sdpsl_feasibility_measures(SDPSLConstraintContainer *ccs,
                                    SDPSLSolution sol, int *N,
                                    MPFRNumber **measures)
    int sdpsl_double_eigenvalues(NumberMatrix A, SDPSLMatrixMask mask, 
                                 double **eigs)
    int sdpsl_gmp_project(SDPSLConstraintContainer *ccs,
       	                  SDPSLSolution sol, SDPSLSolution res)
