#
# Setup for SAGE SDPSL
#

from distutils.core import setup
from distutils.extension import Extension
from Cython.Distutils import build_ext
import os, sys

# Pure Python modules
packages = [ 'sage.sdpsl' ]     # should pick up all Python files listed in __init__.py
package_dir = { 'sage.sdpsl' : 'sage/sdpsl'}
SAGE_ROOT = os.environ['SAGE_ROOT']

INCLUDES = [ '.', 'sage/clib' ] + [ '%s/%s/' % (SAGE_ROOT, x) for x in
                                    [ 'local/include/csage', 
                                      'local/include', 
                                      'local/include/python2.7/',
                                      'devel/sage/sage/ext', 
                                      'devel/sage', 
                                      'devel/sage/sage/gsl',
                                      'devel/sage-main',
                                      'src' ] ]

print INCLUDES

# Extensions (Cython, C(++), the lot)
extensions = [
  Extension('sage.sdpsl.sdpsl',
            sources = ['sage/sdpsl/sdpsl.pyx'],
            libraries = [ 'sagesdpsl', 'mpfsdpsl', 'gmpxx', 'mpfr', 
                          'gmp', 'lapack', 'blas', 'csage', 'm' ],
            language = 'c++',
            include_dirs = INCLUDES,
            library_dirs = [ 'sage/clib', 'lib' ])
        ]

setup(name = 'sage.sdpsl',
    version = '0.85',
    description = 'SDPSL Wrapper for SAGE',
    license = 'GPL v3',
    author = 'Fernando M. de Oliveira Filho',
    author_email = 'fmario AT gmail.com',
    url = 'http://www.ime.usp.br/~fmario',
    cmdclass = { 'build_ext' : build_ext },
    ext_modules = extensions,
    package_dir = package_dir,
    packages = packages
)
