/*
  monomial.cpp

  Copyright (c) 2014 Fernando M. de Oliveira Filho
  <http://www.ime.usp.br/~fmario>
  
  This file is part of SDPSL.
  
  SDPSL is free software: you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your
  option) any later version.
  
  SDPSL is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  for more details.
  
  You should have received a copy of the GNU General Public License
  along with SDPSL.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <sdpsl/monomial.h>

using namespace std;

#ifdef __MPF_BUILD__
namespace mpf_sdp {
#else
namespace sdp {
#endif

Monomial& Monomial::operator*=(const Monomial& other)
{
  if (other.exps.size() > exps.size())
    exps.resize(other.exps.size(), 0);

  for (int i = 0; i < other.exps.size(); i++)
    exps[i] += other.exps[i];

  return *this;
}

Monomial Monomial::operator()(int k)
{
  Monomial ret = *this;

  for (int i = 0; i < exps.size(); i++)
    ret.exps[i] *= k;

  return ret;
}

Monomial operator*(const Monomial& a, const Monomial& b)
{
  Monomial ret = a;
  ret *= b;

  return ret;
}

bool Monomial::operator<(const Monomial& other) const
{
  int max_size = exps.size();
  if (other.exps.size() > max_size) max_size = other.exps.size();

  int i;
  for (i = 0; i < max_size; i++)
    if (operator[](i) != other[i]) break;

  return operator[](i) < other[i];
}


bool Monomial::is_constant() const
{
  for (int i = 0; i < exps.size(); i++)
    if (exps[i]) return false;

  return true;
}

std::ostream& operator<<(std::ostream& out, const Monomial& mon)
{
  bool first = true;

  for (int i = 0; i < mon.exps.size(); i++)
    if (mon.exps[i] != 0) {
      if (!first) out << " * ";
      else first = false;

      out << PolynomialVariable::var_name(i);
      if (mon.exps[i] != 1)
	out << "(" << mon.exps[i] << ")";
    }
  
  return out; 
}

}  // namespace
