/*
  sdpenv.cpp

  Copyright (c) 2014 Fernando M. de Oliveira Filho
  <http://www.ime.usp.br/~fmario>
  
  This file is part of SDPSL.
  
  SDPSL is free software: you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your
  option) any later version.
  
  SDPSL is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  for more details.
  
  You should have received a copy of the GNU General Public License
  along with SDPSL.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <sdpsl/sdpenv.h>
#include <sdpsl/sdperror.h>

#ifdef __MPF_BUILD__
namespace mpf_sdp {
#else
namespace sdp {
#endif

//
// Static variables
//
double SDPEnvironment::global_output_eps = 1e-15;
double SDPEnvironment::global_tolerance_eps = 1e-15;

/*
  set_output_eps
*/
void SDPEnvironment::set_output_eps(double eps)
{
  if (eps < 0)
    throw ValueError(ERROR_MSG("Output epsilon is negative"));

  global_output_eps = eps;
}

/*
  set_tolerance_eps
*/
void SDPEnvironment::set_tolerance_eps(double eps)
{
  if (eps < 0)
    throw ValueError(ERROR_MSG("Tolerance epsilon is negative"));

  global_tolerance_eps = eps;
}

/*
  Constructor
*/
SDPEnvironment::SDPEnvironment(double out_eps, double tol_eps)
{
  if (out_eps < 0)
    throw ValueError(ERROR_MSG("Output epsilon is negative"));
  if (tol_eps < 0)
    throw ValueError(ERROR_MSG("Tolerance epsilon is negative"));

  _output_eps = out_eps;
  _tolerance_eps = tol_eps;
}


}  // namespace
